#!/usr/bin/python
# -*- coding: utf-8 -*-

VISUALIZATIONS_LIST = {
  "Visualizations":
  [{
        "ID": "html-simple",
        "Title": "Html: single-page",
        "Description": "@todo",
      } , {
        "ID": "html-complex",
        "Title": "Html: multi-page",
        "Description": "@todo",
      } , {
        "ID": "markdown",
        "Title": "Markdown: multi-page",
        "Description": "@todo",
      } , {
        "ID": "d3-tree",
        "Title": "D3 Dendogram",
        "Description": "@todo",
      }
  ]
}


BOOTSWATCH_THEMES = ['cerulean'  ,  'flatly' ,  'lumen' ,  'sandstone' ,   'spacelab'  ,  'yeti' , 'cosmo' ,  'darkly' ,  'paper' , 'simplex' , 'superhero' ,   'readable'  ,   'slate' ,  'united']

BOOTSWATCH_THEME_DEFAULT = "simplex"
# 'cyborg' , 'journal'



# !/usr/bin/env python
#  -*- coding: UTF-8 -*-


"""
ONTODOCS
Copyright (c) 2013-2017 __Michele Pasin__ <http://www.michelepasin.org>.
All rights reserved.

More info in the README file.

"""

import time
import click
# http://click.pocoo.org/5/arguments/
# http://click.pocoo.org/5/options/

from ontospy.core import manager as ontospy_manager
from ontospy import VERSION as ontospy_VERSION

from . import *
from .core.builder import *


# http://stackoverflow.com/questions/1714027/version-number-comparison
from distutils.version import StrictVersion



CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument('source', nargs=-1)
@click.option('--outputpath', '-o',  help='Output path (default: home folder).')
@click.option('--title', '-t', help='Title for the visualization (default=graph uri).')
@click.option('--theme',  help='CSS Theme for the html-complex visualization (random=use a random theme).')
@click.option('--showthemes', is_flag=True, help='Show the available CSS theme choices.')
@click.option('--verbose', '-v', is_flag=True, help='Verbose mode.')
def main_cli(source=None, outputpath="", title="", theme="", showthemes=False, verbose=False):
    """
Ontodocs allows to create  documentation for ontologies encoded in RDF/OWL.

E.g.:

> ontodocs http://www.w3.org/2008/05/skos# --theme random -o ~/Desktop/skos

==> generates html docs for the SKOS ontology and save it to your desktop
"""


    if StrictVersion(ontospy_VERSION.replace("v", "")) < StrictVersion('1.8'):
        click.secho("WARNING: OntoDocs requires OntoSpy >= v1.8 but it looks like you have installed %s." % ontospy_VERSION, fg="red")
        sys.exit(0)

    sTime = time.time()
    ontospy_manager.get_or_create_home_repo()
    click.echo(click.style("OntoDocs " + VERSION, bold=True) + click.style(" (OntoSpy " + ontospy_VERSION + ")", fg='white'))

    # click.secho("OntoDocs " + VERSION,  bold=True)
    # click.secho("OntoSpy " + ontospy_VERSION)
    # click.echo(click.style('Hello World!', fg='green') + click.style('Hello World!', fg='red'))
    # click.secho("Local library: '%s'" % get_home_location(), fg='white')
    click.secho("------------", fg='white')

    if showthemes:
        # from .core.builder import show_themes
        show_themes()
        sys.exit(0)

    if theme and theme=="random":
        # from .core.builder import random_theme
        theme = random_theme()

    if outputpath:
        if not(os.path.exists(outputpath)) or not (os.path.isdir(outputpath)):
            click.secho("WARNING: the -o option must include a valid directory path.", fg="red")
            sys.exit(0)

    if not source:
        # ask to show local library
        click.secho("You haven't specified any argument.", fg='red')
        if click.confirm('Select an ontology from the local OntoSpy library?'):
            pass
        else:
            printDebug("Goodbye (tip: use -h for more options).", "comment")
            raise SystemExit(1)


    if source and len(source) > 1:
        click.secho('Note: currently only one argument can be passed', fg='red')

    # note: the local ontospy library gets displayed via this method too
    url = action_visualize(source, fromshell=False, path=outputpath, title=title, theme=theme, verbose=verbose)


    if url:# open browser
        import webbrowser
        webbrowser.open(url)


    # finally: print(some stats.... )
    eTime = time.time()
    tTime = eTime - sTime
    printDebug("\n----------\n" + "Time:	   %0.2fs" %  tTime, "comment")



if __name__ == '__main__':
    try:
        # http://stackoverflow.com/questions/32553969/modify-usage-string-on-click-command-line-interface-on-windows
        main_cli(prog_name='ontodocs')
        sys.exit(0)
    except KeyboardInterrupt as e: # Ctrl-C
        raise e



# !/usr/bin/env python
#  -*- coding: UTF-8 -*-


from . import *  # imports __init__
from .. import *
from ..core.entities import OntoClass, OntoProperty, OntoSKOSConcept, Ontology


# TEMPLATE: HTML SPLITTER MULTI FILE


#
# ===========
# Comments:
# ===========
#





def run(graph, save_on_github=False, main_entity=None):
    """
    From a graph instance outputs some nicely formatted html text.
    
    2016-08-07: hacked for multi file save
    """

    try:
        ontology = graph.ontologies[0]
        uri = ontology.uri
    except:
        ontology = None
        uri = ";".join([s for s in graph.sources])
        
    context = {
                    "ontology": ontology,
                    "main_uri" : uri,
                    "ontograph": graph,
                    "STATIC_PATH": ONTODOCS_VIZ_STATIC,
                }
    

    # Pygments CSS
    try:
        from pygments import highlight
        from pygments.lexers.rdf import TurtleLexer
        from pygments.formatters import HtmlFormatter

        pygments_code = highlight(main_entity.serialize(), TurtleLexer(), HtmlFormatter())
        pygments_code_css = HtmlFormatter().get_style_defs('.highlight')
        context.update({ "pygments_code" : pygments_code,
                         "pygments_code_css": pygments_code_css
                         })
    except Exception as e:
        pass


        
    if type(main_entity) == OntoClass:
        ontotemplate = open(ONTODOCS_VIZ_TEMPLATES + "splitter/splitter_classinfo.html", "r")
        context.update({ "main_entity" : main_entity,
                         "main_entity_type": "class"
                         })
    elif type(main_entity) == OntoProperty:
        ontotemplate = open(ONTODOCS_VIZ_TEMPLATES + "splitter/splitter_propinfo.html", "r")
        context.update({ "main_entity" : main_entity,
                         "main_entity_type": "property"
                         })
    elif type(main_entity) == OntoSKOSConcept:
        ontotemplate = open(ONTODOCS_VIZ_TEMPLATES + "splitter/splitter_conceptinfo.html", "r")
        context.update({ "main_entity" : main_entity,
                         "main_entity_type": "concept"
                         })
    else:
            # if type(main_entity) == Ontology:
        ontotemplate = open(ONTODOCS_VIZ_TEMPLATES + "splitter/splitter_ontoinfo.html", "r")


    t = Template(ontotemplate.read())
    c = Context(context)

    rnd = t.render(c)

    return safe_str(rnd)







if __name__ == '__main__':
    """
    > python -m viz.viz_splitter_multi

    2016-08-04: # testing bypassing the usual abstract routine so to generate multiple files

    """
    import os, sys
    try:

        # script for testing - must launch this module direclty eg

        # > python -m viz.viz_splitter_multi

        func = locals()["run"] # main func dynamically
        # run_test_viz(func)

        import webbrowser, random

        ontouri = get_localontologies()[random.randint(0, 10)] # [13]=foaf #
        print("Testing with URI: %s" % ontouri)

        g = get_pickled_ontology(ontouri)
        if not g:
            g = do_pickle_ontology(ontouri)


        def _saveVizLocally(contents, filename="index.html", path=None):
            if not path:
                filename = ONTODOCS_LOCAL_VIZ + "/" + filename
            else:
                filename = os.path.join(path, filename)

            f = open(filename, 'wb')
            f.write(contents)  # python will convert \n to os.linesep
            f.close()  # you can omit in most cases as the destructor will call it

            url = "file://" + filename
            return url

        from os.path import expanduser
        home = expanduser("~")
        DEST_FOLDER = os.path.join(home, "ontospy-viz")
        if not os.path.exists(DEST_FOLDER):
            os.makedirs(DEST_FOLDER)

        # main index page for graph
        contents = func(g, False, None)
        index_url = _saveVizLocally(contents, "index.html", DEST_FOLDER)
        
        entities = [g.classes, g.properties, g.skosConcepts]
        for group in entities:
            for c in group:
                # getting main func dynamically
                contents = func(g, False, c)
                _filename = c.slug + ".html"
                url = _saveVizLocally(contents, _filename, DEST_FOLDER)

        if index_url:  # open browser
            import webbrowser
            webbrowser.open(index_url)



        sys.exit(0)

    except KeyboardInterrupt as e: # Ctrl-C
        raise e






# !/usr/bin/env python
#  -*- coding: UTF-8 -*-


import os, sys
import json

from ..core import *
from ..core.utils import *
from ..core.builder import *  # loads and sets up Django
from ..core.viz_factory import VizFactory




# ===========
# D3 Bar Hierarchy
# ===========




class D3BarHierarchyViz(VizFactory):
    """
    D3 Bar HierarchyViz

    """


    def __init__(self, ontospy_graph, title=""):
        """
        Init
        """
        super(D3BarHierarchyViz, self).__init__(ontospy_graph, title)
        self.static_files = ["libs/d3-v3", "libs/jquery"]


    def _buildTemplates(self):
        """
        OVERRIDING THIS METHOD from Factory
        """

        jsontree_classes = build_D3treeStandard(0, 99, 1, self.ontospy_graph.toplayer)
        c_total = len(self.ontospy_graph.classes)

        JSON_DATA_CLASSES = json.dumps({'children': jsontree_classes, 'name': 'owl:Thing',})


        extra_context = {
                        "ontograph": self.ontospy_graph,
    					"TOTAL_CLASSES": c_total,
    					'JSON_DATA_CLASSES' : JSON_DATA_CLASSES,
                        }

        # Ontology - MAIN PAGE
        contents = self._renderTemplate("d3/d3_barHierarchy.html", extraContext=extra_context)
        FILE_NAME = "index.html"
        main_url = self._save2File(contents, FILE_NAME, self.output_path)

        return main_url




# if called directly, for testing purposes pick a random ontology

if __name__ == '__main__':

    TEST_ONLINE = False
    try:

        g = get_onto_for_testing(TEST_ONLINE)

        v = D3BarHierarchyViz(g, title="")
        v.build()
        v.preview()

        sys.exit(0)

    except KeyboardInterrupt as e:  # Ctrl-C
        raise e



# !/usr/bin/env python
#  -*- coding: UTF-8 -*-
#
#
# ==================
# VIZ D3 Tree - a d3 dendogram
# ==================


import os, sys
import json

from ..core import *
from ..core.utils import *
from ..core.builder import *  # loads and sets up Django
from ..core.viz_factory import VizFactory





class D3TreeViz(VizFactory):
    """
    D3 viz tree

    """


    def __init__(self, ontospy_graph, title=""):
        """
        Init
        """
        super(D3TreeViz, self).__init__(ontospy_graph, title)
        self.static_files = ["libs/d3-v2", "libs/jquery"]


    def _buildTemplates(self):
        """
        OVERRIDING THIS METHOD from Factory
        """


        c_mylist = build_D3treeStandard(0, 99, 1, self.ontospy_graph.toplayer)
        p_mylist = build_D3treeStandard(0, 99, 1, self.ontospy_graph.toplayerProperties)
        s_mylist = build_D3treeStandard(0, 99, 1, self.ontospy_graph.toplayerSkosConcepts)

        c_total = len(self.ontospy_graph.classes)
        p_total = len(self.ontospy_graph.properties)
        s_total = len(self.ontospy_graph.skosConcepts)


        if False:
            # testing how a single tree would look like
            JSON_DATA_CLASSES = json.dumps(
            {'children' :
                [ {'children' : c_mylist, 'name' : 'Classes', 'id' : "classes" },
                {'children' : p_mylist, 'name' : 'Properties', 'id' : "properties" },
                {'children' : s_mylist, 'name' : 'Concepts', 'id' : "concepts" }],
                'name' : 'Entities', 'id' : "root"
                }
                )

        # hack to make sure that we have a default top level object
        JSON_DATA_CLASSES = json.dumps({'children' : c_mylist, 'name' : 'owl:Thing', 'id' : "None" })
        JSON_DATA_PROPERTIES = json.dumps({'children' : p_mylist, 'name' : 'Properties', 'id' : "None" })
        JSON_DATA_CONCEPTS = json.dumps({'children' : s_mylist, 'name' : 'Concepts', 'id' : "None" })

        extra_context = {
                        "ontograph": self.ontospy_graph,
    					"TOTAL_CLASSES": c_total,
    					"TOTAL_PROPERTIES": p_total,
    					"TOTAL_CONCEPTS": s_total,
    					'JSON_DATA_CLASSES' : JSON_DATA_CLASSES,
    					'JSON_DATA_PROPERTIES' : JSON_DATA_PROPERTIES,
    					'JSON_DATA_CONCEPTS' : JSON_DATA_CONCEPTS,
                        }

        # Ontology - MAIN PAGE
        contents = self._renderTemplate("d3/d3tree.html", extraContext=extra_context)
        FILE_NAME = "index.html"
        main_url = self._save2File(contents, FILE_NAME, self.output_path)

        return main_url




# if called directly, for testing purposes pick a random ontology

if __name__ == '__main__':

    TEST_ONLINE = False
    try:

        g = get_onto_for_testing(TEST_ONLINE)

        v = D3TreeViz(g, title="")
        v.build()
        v.preview()

        sys.exit(0)

    except KeyboardInterrupt as e:  # Ctrl-C
        raise e



# !/usr/bin/env python
#  -*- coding: UTF-8 -*-


##################
#
#  metadata
#
##################



__version__ = "1.1.2"

__copyright__ = "CopyRight (C) 2015-2017 by Michele Pasin"
__license__ = "GNU"
__author__ = "Michele Pasin"
__author_email__ = "michele dot pasin at gmail dot com"

USAGE = "ontodocs [options] [uri]"
VERSION = "v" + __version__

AGENT = "%s/%s" % (__name__, __version__)
